@@ -378,6 +378,22 @@ class TestListOfStrings(TestField):
         self.assertEqual("['abc']", self.field.stringify(['abc']))
 
 
+class TestDictOfListOfStrings(TestField):
+    def setUp(self):
+        super(TestDictOfListOfStrings, self).setUp()
+        self.field = fields.DictOfListOfStringsField()
+        self.coerce_good_values = [({'foo': ['1', '2']}, {'foo': ['1', '2']}),
+                                   ({'foo': [1]}, {'foo': ['1']})]
+        self.coerce_bad_values = [{'foo': [None, None]}, 'foo']
+        self.to_primitive_values = [({'foo': ['1', '2']}, {'foo': ['1', '2']})]
+        self.from_primitive_values = [({'foo': ['1', '2']},
+                                       {'foo': ['1', '2']})]
+
+    def test_stringify(self):
+        self.assertEqual("{foo=['1','2']}",
+                         self.field.stringify({'foo': ['1', '2']}))
+
+
 class TestListOfEnum(TestField):
     def setUp(self):
         super(TestListOfEnum, self).setUp()
